@@ -1,5 +1,6 @@
 using System;
 using System.Collections;
+using System.Collections.Generic;
 using Mono.Unix;
 
 using Gtk;
@@ -18,7 +19,7 @@ namespace Tomboy.Sync
 		private Gtk.Button closeButton;
 		private uint progressBarTimeoutId;
 		
-		private Gtk.ListStore model;
+		private Gtk.TreeStore model;
 		
 		// TODO: Possible to make Tomboy not crash if quit while dialog is up?
 		public SyncDialog ()
@@ -80,10 +81,10 @@ namespace Tomboy.Sync
 			outerVBox.PackStart (progressLabel, false, false, 0);
 			
 			// Create model for TreeView
-			model = new Gtk.ListStore (typeof (string), typeof (string));
+			model = new Gtk.TreeStore (typeof (string), typeof (string));
 			
 			// Create TreeView, attach model
-			Gtk.TreeView treeView = new Gtk.TreeView (model);
+			Gtk.TreeView treeView = new Gtk.TreeView ();
 			treeView.Model = model;
 			
 			// Set up TreeViewColumns
@@ -266,7 +267,7 @@ namespace Tomboy.Sync
 				case SyncState.Locked:
 					Title = Catalog.GetString ("Server Locked");
 					HeaderText = Catalog.GetString ("Server is locked");
-					MessageText = Catalog.GetString ("One of your other computers is currently synchronizing.  Please wait and try again.");
+					MessageText = Catalog.GetString ("One of your other computers is currently synchronizing.  Please wait 2 minutes and try again.");
 					ProgressText = string.Empty;
 					break;
 				case SyncState.PrepareDownload:
@@ -286,8 +287,7 @@ namespace Tomboy.Sync
 					break;
 				case SyncState.Succeeded:
 					int count = 0;
-					foreach (object [] currentRow in model)
-						count++;
+					count += model.IterNChildren ();
 					Title = Catalog.GetString ("Synchronization Complete");
 					HeaderText = Catalog.GetString ("Synchronization is complete");
 					string numNotesUpdated =
@@ -334,7 +334,7 @@ namespace Tomboy.Sync
 					statusText = Catalog.GetString ("Deleted locally");
 					break;
 				case NoteSyncType.DeleteFromServer:
-					statusText = Catalog.GetString ("Deleting from server");
+					statusText = Catalog.GetString ("Deleted from server");
 					break;
 				case NoteSyncType.DownloadModified:
 					statusText = Catalog.GetString ("Updated");
@@ -343,17 +343,20 @@ namespace Tomboy.Sync
 					statusText = Catalog.GetString ("Added");
 					break;
 				case NoteSyncType.UploadModified:
-					statusText = Catalog.GetString ("Uploading changes to server");
+					statusText = Catalog.GetString ("Uploaded changes to server");
 					break;
 				case NoteSyncType.UploadNew:
-					statusText = Catalog.GetString ("Uploading new note to server");
+					statusText = Catalog.GetString ("Uploaded new note to server");
 					break;
 				}
 				AddUpdateItem (noteTitle, statusText);
 			});
 		}
 		
-		void OnNoteConflictDetected (NoteManager manager, Note localConflictNote, NoteUpdate remoteNote)
+		void OnNoteConflictDetected (NoteManager manager,
+		                             Note localConflictNote,
+		                             NoteUpdate remoteNote,
+		                             IList<string> noteUpdateTitles)
 		{
 			SyncTitleConflictResolution savedBehavior = SyncTitleConflictResolution.Cancel;
 			object dlgBehaviorPref = Preferences.Get (Preferences.SYNC_CONFIGURED_CONFLICT_BEHAVIOR);
@@ -363,9 +366,13 @@ namespace Tomboy.Sync
 			SyncTitleConflictResolution resolution = SyncTitleConflictResolution.OverwriteExisting;
 			// This event handler will be called by the synchronization thread
 			// so we have to use the delegate here to manipulate the GUI.
+			// To be consistent, any exceptions in the delgate will be caught
+			// and then rethrown in the synchronization thread.
+			Exception mainThreadException = null;
 			Gtk.Application.Invoke (delegate {
+				try {
 				SyncTitleConflictDialog conflictDlg =
-					new SyncTitleConflictDialog (localConflictNote);
+					new SyncTitleConflictDialog (localConflictNote, noteUpdateTitles);
 				Gtk.ResponseType reponse = Gtk.ResponseType.Ok;
 				
 				bool noteSyncBitsMatch =
@@ -417,7 +424,12 @@ namespace Tomboy.Sync
 			
 				// Let the SyncManager continue
 				SyncManager.ResolveConflict (/*localConflictNote, */resolution);
+				} catch (Exception e) {
+					mainThreadException = e;
+				}
 			});
+			if (mainThreadException != null)
+				throw mainThreadException;
 		}
 
 		#endregion // Private Event Handlers
@@ -443,8 +455,8 @@ namespace Tomboy.Sync
 				NoteArchiver.Instance.GetRenamedNoteXml (note.XmlContent, oldTitle, newTitle);
 			string newCompleteContent = //note.GetCompleteNoteXml ();
 				NoteArchiver.Instance.GetRenamedNoteXml (note.GetCompleteNoteXml (), oldTitle, newTitle);
-			Logger.Debug ("RenameNote: newContent: " + newContent);
-			Logger.Debug ("RenameNote: newCompleteContent: " + newCompleteContent);
+			//Logger.Debug ("RenameNote: newContent: " + newContent);
+			//Logger.Debug ("RenameNote: newCompleteContent: " + newCompleteContent);
 			
 			// We delete and recreate the note to simplify content conflict handling
 			Tomboy.DefaultNoteManager.Delete (note);
@@ -452,8 +464,11 @@ namespace Tomboy.Sync
 			// Create note with old XmlContent just in case GetCompleteNoteXml failed
 			Logger.Debug ("RenameNote: about to create " + newTitle);
 			Note renamedNote = Tomboy.DefaultNoteManager.Create (newTitle, newContent);
-			if (newCompleteContent != null) // TODO: Anything to do if it is null?
-				renamedNote.LoadForeignNoteXml (newCompleteContent);
+			if (newCompleteContent != null) {// TODO: Anything to do if it is null?
+				try {
+					renamedNote.LoadForeignNoteXml (newCompleteContent);
+				} catch {} // TODO: Handle exception in case that newCompleteContent is invalid XML 
+			}
 			if (noteOpen)
 				renamedNote.Window.Present ();
 		}
@@ -465,6 +480,7 @@ namespace Tomboy.Sync
 	public class SyncTitleConflictDialog : Gtk.Dialog
 	{
 		private Note existingNote;
+		private IList<string> noteUpdateTitles;
 		
 		private Gtk.Button continueButton;
 		
@@ -477,14 +493,16 @@ namespace Tomboy.Sync
 		private Gtk.Label headerLabel;
 		private Gtk.Label messageLabel;
 
-		public SyncTitleConflictDialog (Note existingNote) :
+		public SyncTitleConflictDialog (Note existingNote, IList<string> noteUpdateTitles) :
 			base (Catalog.GetString ("Note Conflict"), null, Gtk.DialogFlags.Modal)
 		{
 			this.existingNote = existingNote;
+			this.noteUpdateTitles = noteUpdateTitles;
+			
 			// Suggest renaming note by appending " (old)" to the existing title
 			string suggestedRenameBase = existingNote.Title + Catalog.GetString (" (old)");
 			string suggestedRename = suggestedRenameBase;
-			for (int i = 1; existingNote.Manager.Find (suggestedRename) != null; i++)
+			for (int i = 1; !IsNoteTitleAvailable (suggestedRename); i++)
 				suggestedRename = suggestedRenameBase + " " + i.ToString();
 			
 			VBox outerVBox = new VBox (false, 12);
@@ -556,12 +574,18 @@ namespace Tomboy.Sync
 		private void renameEntry_Changed (object sender, System.EventArgs e)
 		{
 			if (renameRadio.Active &&
-			    existingNote.Manager.Find (RenamedTitle) != null)
+			    !IsNoteTitleAvailable (RenamedTitle))
 				continueButton.Sensitive = false;
 			else
 				continueButton.Sensitive = true;
 		}
 		
+		private bool IsNoteTitleAvailable (string renamedTitle)
+		{
+			return !noteUpdateTitles.Contains (renamedTitle) &&
+				existingNote.Manager.Find (renamedTitle) == null;
+		}
+		
 		// Handler for each radio button's Toggled event
 		private void radio_Toggled (object sender, System.EventArgs e)
 		{
